@@ -1,6 +1,6 @@
 
 (*
- * Copyright (C) 2007-2014 The laby team
+ * Copyright (C) 2007-2020 The laby team
  * You have permission to copy, modify, and redistribute under the
  * terms of the GPL-3.0. For full license terms, see gpl-3.0.txt.
  *)
@@ -32,11 +32,11 @@ let conf_window =
     (F.x "initial window geometry" [])
 
 let conf_window_width =
-  Conf.int ~p:(conf_window#plug "width") ~d:1000
+  Conf.int ~p:(conf_window#plug "width") ~d:800
     (F.x "width of window" [])
 
 let conf_window_height =
-  Conf.int ~p:(conf_window#plug "height") ~d:750
+  Conf.int ~p:(conf_window#plug "height") ~d:600
     (F.x "height of window" [])
 
 exception Error of F.t
@@ -63,9 +63,9 @@ type controls =
       start_vbox: GPack.box;
       start_image: GMisc.image;
       main_hpaned: GPack.paned;
-      menu_quit: GMenu.image_menu_item;
-      menu_home: GMenu.image_menu_item;
-      menu_level: GMenu.image_menu_item;
+      menu_quit: GMenu.menu_item;
+      menu_home: GMenu.menu_item;
+      menu_level: GMenu.menu_item;
       menu_levels: GMenu.menu;
       button_start: GButton.button;
       button_prev: GButton.tool_button;
@@ -75,9 +75,9 @@ type controls =
       button_forward: GButton.toggle_tool_button;
       button_execute: GButton.button;
       map_image: GMisc.image;
-      interprets: GEdit.combo;
-      view_prog: GSourceView2.source_view;
-      view_help: GSourceView2.source_view;
+      interprets: GEdit.combo_box;
+      view_prog: GSourceView3.source_view;
+      view_help: GSourceView3.source_view;
       box_help: GPack.box;
       view_mesg: GText.view;
       view_title: GMisc.label;
@@ -145,11 +145,15 @@ let gtk_init () =
     ant_w_p = pix "ant-w";
   }
 
-let draw_state state ressources (pixmap : GDraw.pixmap) =
+let draw_state state ressources (pixbuf : GdkPixbuf.pixbuf) =
   let size = ressources.size in
   let tile i j p =
-    pixmap#put_pixbuf
-      ~x:(size / 2 + i * size) ~y:(size / 2 + j * size) p
+    let px = size / 2 + i * size in
+    let py = size / 2 + j * size in
+    (* GdkPixbuf.copy_area ~dest:pixbuf ~dest_x:px ~dest_y:py p *)
+    GdkPixbuf.composite ~dest:pixbuf ~alpha:255
+      ~dest_x:px ~dest_y:py ~width:size ~height:size
+      ~ofs_x:(float px) ~ofs_y:(float py) ~scale_x:1.0 ~scale_y:1.0 p
   in
   let i0, j0 = State.pos state in
   let disp_tile i j t =
@@ -175,10 +179,10 @@ let draw_state state ressources (pixmap : GDraw.pixmap) =
   | `None -> ()
   end
 
-let labeled_combo text packing =
+let labeled_combo text packing strings =
   let box = GPack.hbox ~packing () in
   let _ = GMisc.label ~text ~xpad:5 ~ypad:8 ~packing:box#pack () in
-  GEdit.combo ~packing:box#add ()
+  fst (GEdit.combo_box_text ~strings ~packing:box#add ())
 
 let label packing =
   GMisc.label ~ypad:5 ~line_wrap:true ~packing ()
@@ -195,11 +199,11 @@ let label_mesg = F.x "Messages:" []
 let label_help = F.x "Help:" []
 let label_start = F.x "Start" []
 
-let layout () =
+let layout languages =
   let scrolled ?(vpolicy=`ALWAYS) packing =
     GBin.scrolled_window ~packing ~hpolicy:`AUTOMATIC ~vpolicy ()
   in
-  let monofont = Pango.Font.from_string "monospace" in
+  let monofont = GPango.font_description_from_string "monospace" in
   let window = GWindow.window ~resizable:true () in
   let windowbox = GPack.vbox ~packing:window#add () in
   let menu_bar = GMenu.menu_bar ~packing:windowbox#pack () in
@@ -207,15 +211,15 @@ let layout () =
   let sub_main = GMenu.menu () in
   let menu_main = GMenu.menu_item ~label:(Fd.render_raw label_menu)
     ~packing:menu_bar#append () in
-  let menu_level = GMenu.image_menu_item
+  let menu_level = GMenu.menu_item
     ~label:(Fd.render_raw label_level) ~packing:menu_bar#append () in
-  let menu_fullscreen = GMenu.image_menu_item ~stock:`FULLSCREEN
+  let menu_fullscreen = GMenu.menu_item ~label:"Fullscreen"
     ~packing:sub_main#append () in
-  let menu_unfullscreen = GMenu.image_menu_item ~stock:`LEAVE_FULLSCREEN
+  let menu_unfullscreen = GMenu.menu_item ~label:"Leave Fullscreen"
     ~packing:sub_main#append ~show:false () in
-  let menu_quit = GMenu.image_menu_item ~stock:`QUIT
+  let menu_quit = GMenu.menu_item ~label:"Quit"
     ~packing:sub_main#append () in
-  let menu_home = GMenu.image_menu_item ~stock:`HOME
+  let menu_home = GMenu.menu_item ~label:"Home"
     ~packing:sub_main#append () in
   let fullscreen () =
     menu_fullscreen#misc#hide ();
@@ -241,17 +245,18 @@ let layout () =
   let _ = GMisc.label ~markup:(Fd.render_raw label_welcome)
     ~justify:`CENTER ~packing:mstart_vbox#pack () in
   let interprets =
-    labeled_combo (Fd.render_raw label_language) mstart_vbox#pack
+    labeled_combo (Fd.render_raw label_language) mstart_vbox#pack languages
   in
   let button_start = GButton.button ~packing:mstart_vbox#pack
     ~label:(Fd.render_raw label_start) () in
 
   (* Game screen *)
   let hpaned = GPack.paned `HORIZONTAL ~packing:main_vbox#add () in
-  hpaned#set_position 620;
+  let tile_size = max 5 conf_tilesize#get in
+  hpaned#set_position (80 + 550 * tile_size / 40);
   let lvbox = GPack.vbox ~packing:hpaned#add1 () in
   let vpaned = GPack.paned `VERTICAL ~packing:hpaned#add () in
-  vpaned#set_position 450;
+  vpaned#set_position 400;
   let view_title = label lvbox#pack in
   let view_comment = label lvbox#pack in
   let sw_laby = scrolled ~vpolicy:`AUTOMATIC lvbox#add in
@@ -259,7 +264,7 @@ let layout () =
   label_txt (Fd.render_raw label_help) box_help#pack;
   let sw_help = scrolled box_help#pack in
   let view_help =
-    GSourceView2.source_view ~editable:false ~packing:sw_help#add ()
+    GSourceView3.source_view ~height:100 ~editable:false ~packing:sw_help#add ()
   in
   view_help#set_indent 1;
   view_help#misc#modify_font monofont;
@@ -267,24 +272,24 @@ let layout () =
   label_txt (Fd.render_raw label_prog) rtvbox#pack;
   let sw_prog = scrolled rtvbox#add in
   let view_prog =
-    GSourceView2.source_view
+    GSourceView3.source_view
       ~auto_indent:true ~indent_width:2 ~insert_spaces_instead_of_tabs:true
       ~show_line_numbers:true ~packing:sw_prog#add ()
   in
   view_prog#set_indent 1;
   view_prog#misc#modify_font monofont;
+  let bbox = GPack.hbox ~packing:rtvbox#pack () in
+  let button_execute = GButton.button ~packing:bbox#pack ~stock:`EXECUTE () in
   let rbvbox = GPack.vbox ~packing:vpaned#add2 () in
-  let toolbar = GButton.toolbar ~packing:rbvbox#pack ~style:`BOTH () in
+  let toolbar = GButton.toolbar ~packing:bbox#pack () in
   let button stock = GButton.tool_button ~packing:toolbar#insert ~stock () in
   let tbutton stock =
     GButton.toggle_tool_button ~packing:toolbar#insert ~stock ()
   in
-  let sti = GButton.separator_tool_item in
-  let button_prev = button `GO_BACK in
-  let button_next = button `GO_FORWARD in
-  let _ = sti ~expand:true ~draw:false ~packing:toolbar#insert () in
   let button_backward = tbutton `MEDIA_REWIND in
+  let button_prev = button `GO_BACK in
   let button_play = tbutton `MEDIA_PLAY in
+  let button_next = button `GO_FORWARD in
   let button_forward = tbutton `MEDIA_FORWARD in
   view_prog#misc#grab_focus ();
   label_txt (Fd.render_raw label_mesg) rbvbox#pack;
@@ -292,8 +297,6 @@ let layout () =
   let view_mesg = GText.view ~editable:false ~packing:sw_mesg#add  () in
   view_mesg#misc#modify_font monofont;
   let map_image = GMisc.image ~packing:sw_laby#add_with_viewport () in
-  let bbox = GPack.hbox ~packing:rtvbox#pack ~homogeneous:true () in
-  let button_execute = GButton.button ~packing:bbox#pack ~stock:`EXECUTE () in
   button_execute#set_focus_on_click false;
   {
     window = window;
@@ -315,10 +318,10 @@ let layout () =
     view_title = view_title; view_comment = view_comment;
   }
 
-let make_pixmap tile_size level =
+let make_pixbuf tile_size level =
   let sizex, sizey = Level.size level in
   let width, height = tile_size * (1 + sizex), tile_size * (1 + sizey) in
-  GDraw.pixmap ~width ~height ()
+  GdkPixbuf.create ~width ~height ~has_alpha:true ()
 
 
 let display_gtk ressources =
@@ -337,20 +340,19 @@ let display_gtk ressources =
     ]
   );
 
-  let bg = ref `WHITE in
-  let c = layout () in
+  let c = layout language_list in
   let level_load name =
     let l = Level.load (Res.get ["levels"; name]) in
-    c.map_image#set_pixmap (make_pixmap ressources.size l); l
+    c.map_image#set_pixbuf (make_pixbuf ressources.size l); l
   in
   let syntaxd = Res.get ["syntax"] in
   let add_search_path m l = m#set_search_path (l @ m#search_path) in
-  let ssm = GSourceView2.source_style_scheme_manager true in
+  let ssm = GSourceView3.source_style_scheme_manager true in
   add_search_path ssm [syntaxd; Res.path [syntaxd; "styles"]];
   let style = ssm#style_scheme conf_source_style#get in
   c.view_prog#source_buffer#set_style_scheme style;
   c.view_help#source_buffer#set_style_scheme style;
-  let slm = GSourceView2.source_language_manager false in
+  let slm = GSourceView3.source_language_manager false in
   add_search_path slm [syntaxd; Res.path [syntaxd; "language-specs"]];
 
   (* gui outputs *)
@@ -369,12 +371,10 @@ let display_gtk ressources =
     end
   in
   let draw image state =
-    let p : GDraw.pixmap = image#pixmap in
-    p#set_foreground !bg;
-    let width, height = p#size in
-    p#rectangle ~x:0 ~y:0 ~width ~height ~filled:true ();
+    let p : GdkPixbuf.pixbuf = image#pixbuf in
+    GdkPixbuf.fill p 0l;
     draw_state state ressources p;
-    image#set_pixmap p
+    image#set_pixbuf p
   in
 
   (* game creation *)
@@ -432,21 +432,23 @@ let display_gtk ressources =
     play_inactive ();
   in
   let setupmod () =
-    let name = c.interprets#entry#text in
-    begin match List.mem name language_list with
-    | true ->
-	let lmod = List.find (fun x -> x#name = name) mods in
-	c.view_prog#buffer#set_text (command#chg_mod lmod);
-	let l = slm#language name in
-	if l = None then
-	  log#warning (
+    begin match
+      try Some (List.nth language_list c.interprets#active)
+      with _ -> None
+    with
+      | Some name ->
+	 let lmod = List.find (fun x -> x#name = name) mods in
+	 c.view_prog#buffer#set_text (command#chg_mod lmod);
+	 let l = slm#language name in
+	 if l = None then
+	   log#warning (
 	    F.x "cannot load syntax for <name> mod" [
 	      "name", F.string name;
 	    ]
 	  );
-	c.view_prog#source_buffer#set_language l;
-	c.view_help#source_buffer#set_language l;
-    | false -> ()
+	 c.view_prog#source_buffer#set_language l;
+	 c.view_help#source_buffer#set_language l;
+      | None -> ()
     end
   in
   let newmod () =
@@ -505,10 +507,9 @@ let display_gtk ressources =
     GMain.Main.quit ()
   in
   let altdestroy _ = destroy (); true in
-  c.interprets#set_popdown_strings language_list;
   let smod = Mod.conf_selected#get in
-  if List.mem smod language_list
-  then c.interprets#entry#set_text smod;
+  let select i m = if m = smod then c.interprets#set_active i in
+  List.iteri select language_list;
 
   let group = ref None in
   let add_language l =
@@ -532,7 +533,7 @@ let display_gtk ressources =
   ignore (c.button_backward#connect#toggled ~callback:backward_cb);
   ignore (c.button_forward#connect#toggled ~callback:forward_cb);
   ignore (c.button_execute#connect#clicked ~callback:execute);
-  ignore (c.interprets#entry#connect#changed ~callback:newmod);
+  ignore (c.interprets#connect#changed ~callback:newmod);
   ignore (c.view_prog#buffer#connect#changed ~callback:show_execute);
   ignore (c.menu_quit#connect#activate ~callback:destroy);
   ignore (c.menu_home#connect#activate ~callback:exit_play);
@@ -541,12 +542,10 @@ let display_gtk ressources =
   exit_play ();
   c.window#set_default_size conf_window_width#get conf_window_height#get;
   c.window#show ();
-  (* bg color has to be retrieved after c.window#show *)
-  bg := `COLOR (c.window#misc#style#light `NORMAL);
   if List.mem "0.laby" levels_list
   then newlevel "0.laby"
   else if levels_list <> [] then newlevel (List.hd levels_list);
-  c.start_image#set_pixmap (make_pixmap ressources.size Level.dummy);
+  c.start_image#set_pixbuf (make_pixbuf ressources.size Level.dummy);
   draw c.start_image (!start_animation);
   ignore (GMain.Main.main ())
 
